#include <string.h>
#include <stdio.h>
#include "stdlib.h"
#include "ctype.h"
#include "cmdline.h"
#include "board.h"
#ifdef USE_CMDLINE_RTC
#include "rtc.h"
#endif

#if defined(USE_CMDLINE)

#define MAX_COMMAND_PARS 6
#ifdef USE_CMDLINE_RTC
#define PRINT_CMD() printf("%02d:%02d:%02d>", RTC_TimeStruct.Hours, RTC_TimeStruct.Minutes, RTC_TimeStruct.Seconds)
#else
#define PRINT_CMD() printf("cmd>")
#endif


char buffer[128];    // buffer pro radek
unsigned char bLineIndex=0, CmdReady = 0;
unsigned char CMDLINE_DBG(tPar *pPar, unsigned char ucParNum, unsigned char ucSwitch);
uint8_t rxData;	/* Byte received from DBG terminal */

tCOMMAND Commands[MAX_COMMANDS];
unsigned char ucCmdIndex=0;
#define printinfoline() printf("---(FW:%s [%s %s] ver.: %s)---\r\n", FW_NAME, __DATE__, __TIME__, FW_VERSION)

void CmdLine_Init(void)
{
	HAL_UART_Receive_DMA(&CMDLINE_UART_BUS_HANDLER, &rxData, 1);
	CmdLine_RegisterCmd("h", "", CMDLINE_DBG, 0);
	CmdLine_RegisterCmd("boot", "", CMDLINE_DBG, 1);
	printinfoline();
}

unsigned char CMDLINE_DBG(tPar *pPar, unsigned char ucParNum, unsigned char ucSwitch)
{
	unsigned char ucI;
	switch(ucSwitch)
	{
	case 0 :	//debug command
		printinfoline();//printf("---(%s [%s %s] ver.: %s)---\r\n", FW_NAME, __DATE__, __TIME__, FW_VERSION);
		if (*((unsigned long *)0x20003FF0) == 0xDEADDEAD) printf("ST-LINK-VERSION\r\n");
		for (ucI=0; ucI<MAX_COMMANDS; ucI++)
		{
			if (Commands[ucI].pCmdName == NULL) return 0xFF;
			else printf(" %s %s\r\n",(char *)Commands[ucI].pCmdName, (char *)Commands[ucI].pCmdPar);
		}
		break;
	case 1 :	//debug command
		*((unsigned long *)0x20003FF0) = 0xDEADBEEF;
		for (ucI=0; ucI<6; ucI++)
		{
			printf("Close terminal in (%d)s\r\n", 6-ucI);
			//while (Delay_1s_Expired() == 0);
		}
		//NVIC_SystemReset();
	}//switch
	return 0;
}

unsigned char CmdLine_SaveByte(unsigned char ucByte)
{
	if (CmdReady == 1) return ERRC_FALSE;
	if ((ucByte == '\r') || (ucByte == '\n'))
	//if ((ucByte == '\r') && (buffer[bLineIndex] == '\n'))
	{
		buffer[bLineIndex++] = '\0';
		bLineIndex = 0;
		CmdReady = 1;	//set if command ready
		return ERRC_OK;
	}
	else
	{
		buffer[bLineIndex++] = ucByte;
		return ERRC_FALSE;
	}

}

/*unsigned int CmdLineHandler(MESSAGE *msg);
unsigned char CmdLine_Find(MESSAGE *msg );
*/
unsigned char CmdLine_GetVersion()
{
  return (unsigned char)CMDLINE_VERSION;
}

unsigned  CmdLine_GetNumber(char *pStringNumber, unsigned *Number)
{
  char cCh;
  unsigned uNumber=0, uDigit;
  unsigned char ucI=0, ucScale=10;
  
  if ((pStringNumber[0]=='0') && (pStringNumber[1]=='x')) //HEX number
  {
    ucScale=16;
    ucI=2;
  }
  
  do
  {
    cCh=pStringNumber[ucI];
    if (cCh>='0' && cCh<='9') uDigit= cCh - '0';
    else if (cCh>='A' && cCh<='F') uDigit= cCh - 'A' + 10;
    else if (cCh>='a' && cCh<='f') uDigit= cCh - 'a' + 10;
    else
    {
      uDigit = 0xFF;
    }
    if (uDigit>=ucScale) return ERRC_CMD_LINE_GET_NUMBER_ERR | ERRC_INVALID_PARAMETER;
    uNumber=uNumber*ucScale + uDigit;
    ucI++;
  } while (pStringNumber[ucI]!='\0');
  *Number=uNumber;
  return ERRC_OK;
}


unsigned char CmdLine_GetParameters(char *buffer, 
                                    unsigned char ucCmdIndex,
                                    tPar *pPar)
{
  unsigned char Err;
  char pStringPar[MAX_STRING_LENGTH];
  unsigned char ucI,ucK,ucJ=0, ucParIndex=0,ucParCnt=0;

  //for (ucI=0; ucI<MAX_STRING_LENGTH; ucI++) pStringPar[ucI]=0;
  memset(pStringPar, 0, MAX_STRING_LENGTH);
  for (ucI=1+Commands[ucCmdIndex].ucCmdNameLength; ucI<1+strlen(buffer); ucI++)
  {
    if ((buffer[ucI]!=' ') && (buffer[ucI]!='\0')) 
    {
      pStringPar[ucJ]=buffer[ucI];
      ucJ++;
    }
    else 
    {
      //Check if number of pars are valid
      if (++ucParCnt > Commands[ucCmdIndex].ucCmdParLength)
        return ERRC_CMD_LINE_GET_PARAMETERS | ERRC_INVALID_PARAMETER_NUMBER;
      switch(Commands[ucCmdIndex].pCmdPar[ucParIndex])
      {
      case 'x': 
         Err = CmdLine_GetNumber(&pStringPar[0], &(pPar[ucParIndex++].x));
         if (Err) return Err;
        break;
      case 's':
        //Check String length
        if (strlen(&pStringPar[0]) > MAX_STRING_LENGTH) 
          return ERRC_CMD_LINE_GET_PARAMETERS | ERRC_INVALID_STRING_LENGTH;
        
        for (ucK=0; ucK<MAX_STRING_LENGTH; ucK++) 
          pPar[ucParIndex].s[ucK] = pStringPar[ucK];
        ucParIndex++;
        break;
      case 'd':
        Err = CmdLine_GetNumber(&pStringPar[0],(unsigned int *)&(pPar[ucParIndex++].d));
        if (Err) return Err;
        break;
      default:
        break;
      }
      
      //for (ucJ=0; ucJ<MAX_STRING_LENGTH; ucJ++) pStringPar[ucJ]=0;
      memset(pStringPar, 0, MAX_STRING_LENGTH);
      ucJ=0;
    }
  }
  if (ucParCnt < Commands[ucCmdIndex].ucCmdParLength)
    return ERRC_CMD_LINE_GET_PARAMETERS | ERRC_INVALID_PARAMETER_NUMBER;
 
  return ERRC_OK;
}
unsigned char CmdLine_Find(char *buffer )
{
  char *cTmp=NULL;
  char pCmd[MAX_COMMANDS];
  unsigned char ucI,ucJ;

  memset(pCmd, 0, MAX_COMMANDS);
  cTmp=strchr((char *) buffer,' '); //Find SPACE in received Line
  ucJ=cTmp - (char *)buffer;
  if (ucJ>8) ucJ=8;
  if (cTmp!=NULL) 
  {
    strncpy(pCmd,(char *) buffer,ucJ);
    pCmd[ucJ]='\0';
  }
  else
  {
    strncpy(pCmd,(char *) buffer,8);
  }
  for (ucI=0; ucI<MAX_COMMANDS; ucI++)
  {
	if (Commands[ucI].pCmdName == NULL) return 0xFF;
    if (strcmp(pCmd,(char *)Commands[ucI].pCmdName)==0) return ucI;
  }
  return 0xFF;
}

unsigned char CmdLineHandler()
{
  unsigned char ucCmdIndex;
  unsigned char Err;
  tPar test[MAX_COMMAND_PARS];
#if defined(USE_CMDLINE_RTC)
  RTC_TimeTypeDef RTC_TimeStruct;
  HAL_RTC_GetTime(&hrtc, &RTC_TimeStruct, RTC_FORMAT_BIN);
#endif
  if (CmdReady == 0) return ERRC_FALSE;
  ucCmdIndex = CmdLine_Find(buffer); //Find Command in database
  if (ucCmdIndex==0xFF) 
  {
	printf("Unknown Command...\r\n");
	PRINT_CMD();
	CmdReady = 0;
    return ERRC_CMD_LINE_HANDLER_ERR | ERRC_COMMAND_NOT_FOUND;
  }
  //ucTmp = ucCmdIndex;
  Err = CmdLine_GetParameters(buffer, ucCmdIndex, test);
  if (Err) printf("Syntax Error...\r\n");
  else Commands[ucCmdIndex].pCallBack(test,0,Commands[ucCmdIndex].ucSwitch);
  PRINT_CMD();
  CmdReady = 0;
  return ERRC_OK;

}//CmdLineHandler



unsigned char CmdLine_RegisterCmd( char *pCmdName, 
                          //  unsigned char ucCmdNameLength,
                            char *pCmdPar,
                          //  unsigned char ucCmdParLength,
                            tCallBack0 *pCallBack,
                            unsigned char ucSwitch)
{
  unsigned char ucI;
  
  //Find if command hasn't already registered
  for (ucI=0; ucI<MAX_COMMANDS;ucI++)       
    if (Commands[ucI].pCmdName == pCmdName) 
      return ERRC_CMD_LINE_REGISTER_CMD_ERR | ERRC_COMMAND_ALREADY_EXIST;

  for (ucI=0; ucI<strlen((char *)pCmdPar); ucI++)
  {
    switch (pCmdPar[ucI])
    {
    case 'x':      break;
    case 'd':      break;
    case 's':      break;
    default:       return ERRC_CMD_LINE_REGISTER_CMD_ERR | ERRC_INVALID_PARAMETER_TYPE;
    }
  }
  
  //Check if Command Database is full
  if (ucCmdIndex>MAX_COMMANDS) return ERRC_CMD_LINE_REGISTER_CMD_ERR | ERRC_TOO_MANY_COMMANDS;
  
  //Register Command
  Commands[ucCmdIndex].pCmdName = pCmdName;
  Commands[ucCmdIndex].ucCmdNameLength = strlen((char *)pCmdName);
  Commands[ucCmdIndex].pCmdPar = pCmdPar;
  Commands[ucCmdIndex].ucCmdParLength = strlen((char *)pCmdPar);
  Commands[ucCmdIndex].pCallBack = pCallBack;
  Commands[ucCmdIndex].ucSwitch = ucSwitch;
  ucCmdIndex++;
  return 0;
}

#endif
