/**
  ******************************************************************************
  * @file    lightsense.c
  * @author  [ELKYS servis s.r.o.] Application Team
  * @brief   This file provides a set of functions needed to manage the light sensor
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 ELKYS Servis s.r.o. (Czech Republic)
  * All rights reserved.</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of [ELKYS servis s.r.o.] nor the names of other
  *    contributors to this software may be used to endorse or promote products
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for [ELKYS servis s.r.o.].
  * 5. Redistribution and use of this software other than as permitted under
  *    this license is void and will automatically terminate your rights under
  *    this license.
  *
  * THIS SOFTWARE IS PROVIDED BY ELKYS SERVIS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT
  * SHALL ELKYS SERVIS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "configuration.h"
#include "lightsense.h"

static LIGHTSENSE_DrvTypeDef *LightsenseDrv;


/**
  * @brief  Initialize the LIGHTSENSE.
  * @retval LIGHTSENSE_OK or LIGHTSENSE_ERROR
  */
LIGHTSENSE_StatusTypeDef BSP_LIGHTSENSE_Init(uint8_t i2caddr)
{  
	LIGHTSENSE_StatusTypeDef ret = LIGHTSENSE_OK;
	LIGHTSENSE_InitTypeDef APDS9301_InitStructure;
	uint16_t ctrl = 0x0000;
	if((Apds9301LightSensorDrv.ReadID(i2caddr) & APDS9301_WHO_AM_I_PARTNUMBER_MASK) != APDS9301_WHO_AM_I)
	{
	  ret = LIGHTSENSE_ERROR;
	}
	else
	{
		/* Initialize driver structure */
		LightsenseDrv = &Apds9301LightSensorDrv;
		/* Configure the main parameters */
		/*
		 * APDS9301_REG_CONTROL = LSB
		 * APDS9301_REG_TIMING = MSB
		 */
		APDS9301_InitStructure.Power_Mode = APDS9301_BIT_POWERUP;
		ctrl = ((APDS9301_BIT_GAIN_16X | APDS9301_BIT_INTEG_402MS) << 8) | APDS9301_InitStructure.Power_Mode;
		LightsenseDrv->Init(i2caddr, ctrl);
	}
	return ret;
}

/**
  * @brief  DeInitialize the LIGHTSENSE.
  * @retval None.
  */
void BSP_LIGHTSENSE_DeInit(uint8_t i2caddr)
{
  /* DeInitialize the accelerometer IO interfaces */
  if(LightsenseDrv != NULL)
  {
    if(LightsenseDrv->DeInit != NULL)
    {
    	LightsenseDrv->DeInit(i2caddr);
    }
  }
}

/**
  * @brief  Set/Unset the LIGHTSENSE in low power mode.
  * @param  status 0 means disable Low Power Mode, otherwise Low Power Mode is enabled
  * @retval None
  */
void BSP_LIGHTSENSE_LowPower(uint8_t i2caddr, uint16_t status)
{
  /* Set/Unset the ACCELERO in low power mode */
  if(LightsenseDrv != NULL)
  {
   	LightsenseDrv->LowPower(i2caddr, status);
   }
}

/**
  * @brief  Get values.
  * @param  pDataIR Pointer on 2 lights table with
  *                  pDataIR[0] = IR1, pDataIR[1] = IR2
  * @retval None
  */
void BSP_LIGHTSENSE_GetValue(uint8_t i2caddr, uint16_t *pDataIR)
{
  if(LightsenseDrv != NULL)
  {
    if(LightsenseDrv->GetValue != NULL)
    {   
    	LightsenseDrv->GetValue(i2caddr, pDataIR);
    }
  }
}

/************************ (C) COPYRIGHT ELKYS servis s.r.o. *****END OF FILE****/
