/**
  ******************************************************************************
  * File Name       : nmea.h
  * Description     : This file contains all the functions prototypes for
  *                   the nmea driver.
  * Author          : ELKYS servis s.r.o. (2019)
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2019 ELKYS servis s.r.o.
  * All rights reserved.</center></h2>
  *
  ******************************************************************************
  */
#ifndef _DRV_NMEA_H_
#define _DRV_NMEA_H_

#include "buffer.h"
#ifdef USE_CMDLINE
#include <stdio.h>
#include "cmdline.h"
#endif
extern uint8_t NMEA_LineComplete;
extern uint8_t rxGNSS; /* receive data from GNSS */
typedef enum
{
	GNSS_MODE_OFF = 0x00,   	/* GNSS Device is Off */
	GNSS_MODE_FIXED = 0x01,   	/* Signal Fixed */
	GNSS_MODE_TRACKING = 0x02,	/* Signal Tracking */
} GNSSMode_TypeDef;

typedef struct
{
  uint8_t SatsInView_GPS;
  uint8_t SatsInView_GLS;
  uint8_t RxMessage[80];
  uint8_t RxMessageIndex;
  //Time
  uint8_t hour;
  uint8_t min;
  uint8_t sec;

  uint8_t Fix;
  uint8_t Fix_prev;
  uint8_t DataValid;
  uint8_t DataValid_prev;
  uint8_t Latitude_dd;
  uint32_t Latitude_mm;
  float Latitude;
  char LatitudeNS;
  char LongitudeEW;
  uint8_t Longitude_dd;
  uint32_t Longitude_mm;
  float Longitude;
  uint8_t SatsUsed;
  uint8_t HDOP;
  uint8_t Altitude;
  uint32_t Speed_A;
  uint32_t Speed_B;
  GNSSMode_TypeDef Mode;
}GNSSInfo_TypeDef;

typedef struct
{
	uint8_t RxDataIndex;
	uint8_t RxData[80];
}GNSSMsg_TypeDef;

extern GNSSInfo_TypeDef GNSSInfo;

#define USE_TXT
#define NMEA_FLAG	0x24

void NMEA_Init(void);
void NMEA_ToggleON(void);
void NMEA_Decode(BufferItem_TypeDef* Msg);
void NMEA_Handler(void);
void NMEA_Logger(void);
void NMEA_StoreByte(uint8_t* ucByte);


#ifdef USE_GPS_COMMAND
#ifdef USE_CMDLINE
unsigned char GPS_DBG(tPar *pPar, unsigned char ucParNum, unsigned char ucSwitch);
#endif
#endif


#endif
